/*{ ttc_gpio.c ***********************************************

 Written by Gregor Rebel 2010-2012

 Architecture independent support for General Purpose Input Output (GPIO).

 Currently implemented architectures: stm32

}*/

#include "ttc_gpio.h"

/* functions defined as macros in ttc_gpio.h (faster)

bool ttc_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin);
void ttc_gpio_setn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);
u8_t ttc_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin);
void ttc_gpio_from_index8(u8_t PhysicalIndex, ttc_GPIO_BANK* GPIOx, u8_t* Pin);

Synonyms:

bool gget(GPIO_TypeDef* GPIOx, u8_t Pin);
void gset(GPIO_TypeDef* GPIOx, u8_t Pin);
void gclr(GPIO_TypeDef* GPIOx, u8_t Pin);
void gsetn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);

*/

void ttc_gpio_init2_variable(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {

#ifdef TARGET_ARCHITECTURE_STM32
    #define TTC_INIT_PORT2
    stm32_gpio_variable(Port, GPIOx, Pin);
    stm32_gpio_init_variable( Port, stm32_map_Type(Type), stm32_map_Speed(Speed) );
#endif

#ifdef TARGET_ARCHITECTURE_EM357
    #define TTC_INIT_PORT2
    em357_loadPort_variable(Port, GPIOx, Pin);
    em357_initPort2_variable( Port, em357_map_Type(Type), em357_map_Speed(Speed) );
#endif

#ifndef TTC_INIT_PORT2
  #warning Missing low level implementation for ttc_gpio_init2_variable()
#endif
}
void ttc_gpio_init2(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {
#ifdef TARGET_ARCHITECTURE_STM32
    #define TTC_GPIO_INIT2
    stm32_gpio_init(GPIOx, Pin, stm32_map_Type(Type), stm32_map_Speed(Speed) );
#endif

#ifdef TARGET_ARCHITECTURE_EM357
    #define TTC_GPIO_INIT2
    em357_gpio_init(GPIOx, Pin, stm32_map_Type(Type), stm32_map_Speed(Speed) );
#endif

#ifndef TTC_GPIO_INIT2
  #warning Missing low level implementation for ttc_gpio_init2_variable()
#endif

}
void ttc_gpio_init(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type) {
    ttc_gpio_init2(GPIOx, Pin, Type, tgs_Max);
}
